//
// Author: Jon Trulson <jon@radscan.com>
// Copyright (c) 1994-2018 Jon Trulson
//
// The MIT License
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
// BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
// ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
//

#ifndef TORP_H_INCLUDED
#define TORP_H_INCLUDED

#include "conqdef.h"
#include "team.h"

// make it int sized to preserve CB compatibility.
enum TorpStatus : int {
    TS_OFF = 1,                 /* ready for use */
    TS_LAUNCHING = 2,           /* being created */
    TS_LIVE = 3,                /* armed and flying */
    TS_DETONATE = 4,            /* detonate signal sent */
    TS_FIREBALL = 5,            /* exploding, needed for display() */
    TS_RESERVED = 6             /* client reserve slot flag */
};

typedef struct
{
    TorpStatus status;          /* one of the TS_ values */
    int fuse;                   /* seconds left */
    real x;                     /* x and y coordinates of torps */
    real y;
    real dx;                    /* x and y increments */
    real dy;                    /*  in mega-meters per iter seconds */
    real mult;                  /* multiplier when detonated */
    int war[NUM_PLAYERTEAMS];    /* copy of war when fired */
} Torp_t;

#endif /* TORP_H_INCLUDED */
